const METHOD_NOT_IMPLEMENTED = 'Method not implemented';
const DIFFERENT_TYPES = 'Schema validation failure: Different types.';
const ObjectId = require('mongodb').ObjectId;

class BaseCollection {
    constructor() {
        this.collection = BaseCollection.db.collection(this.getCollection());
    }

    save() {
        if (this._id === undefined) {
            return this.create()
        }

        return this.update();
    }

    async update() {
        const schema = this.getSchema();
        const finalDocument = {};
        const schemaKeys = Object.keys(schema);


        schemaKeys.forEach((value) => {
            if (!this.areSameType(this[value], schema[value])) {
                throw DIFFERENT_TYPES;
            }

            finalDocument[value] = this[value];
        });

        const oId = new ObjectId(this._id);
        // TODO: Error handling
        const {result} = await this.collection.updateOne({'_id': oId}, {$set: finalDocument});

        return result
    }

    async create() {
        const schema = this.getSchema();
        const finalDocument = {};
        const schemaKeys = Object.keys(schema);

        schemaKeys.forEach((value) => {
            if (!this.areSameType(this[value], schema[value])) {
                throw DIFFERENT_TYPES;
            }

            finalDocument[value] = this[value];
        });

        const {result} = await this.collection.insert(finalDocument);

        this._id = result.ops[0]._id;

        return result;
    }

    async find(query) {
        return await this.collection.find(query).toArray();
    }

    areSameType(val1, val2) {
        return typeof val1 === typeof val2;
    }

    getSchema() {
        throw METHOD_NOT_IMPLEMENTED;
    }

    getCollection() {
        throw METHOD_NOT_IMPLEMENTED;
    }
}

BaseCollection.setDb = function(db) {
    BaseCollection.db = db;
};

module.exports = BaseCollection;
