const MongoClient = require('mongodb').MongoClient;
const assert = require('assert');

// Connection URL
let url;

async function startServer() {
    return new Promise(resolve =>
        // Use connect method to connect to the server
        MongoClient.connect(url, (err, client) => {
            resolve({err, client});
        })
    );
}

async function configureServer(urlParam = 'mongodb://localhost:27017') {
    url = urlParam;
    const {err, client} = await startServer();

    assert.equal(null, err);
    console.log(`Connected successfully to the database on port ${url.split(':')[2]}`);

    return client;
}

function mockDb(db) {
    const oldCollection = db.collection.bind(db);

    db.collection = function(collectionName) {
        const collection = oldCollection(collectionName);

        mockInsert(collection);
        mockFind(collection);
        mockUpdate(collection);

        return collection;
    }
}

function mockInsert(collection) {
    const oldInsert = collection.insert.bind(collection);

    collection.insert = function(document) {
        return new Promise(resolve => {
            oldInsert(document, (err, result) => {
                resolve({
                    err,
                    result
                });
            })
        });
    }
}

function mockUpdate(collection) {
    const oldUpdate = collection.updateOne.bind(collection);

    collection.updateOne = function(query, document) {
        return new Promise(resolve => {
            oldUpdate(query, document, (err, result) => {
                resolve({
                    err,
                    result
                });
            })
        });
    }
}

function mockFind(collection) {
    const oldFind = collection.find.bind(collection);

    collection.find = function(query) {
        const resFind = oldFind(query);
        const oldToArray = resFind.toArray.bind(resFind);

        resFind.toArray = function() {
            return new Promise(resolve => {
                oldToArray((err, result) => {
                    resolve({
                        err,
                        result
                    });
                })
            });
        };

        return resFind;
    }
}

module.exports = {
    configureServer,
    mockDb
};
